#ifndef LIBRARY_H_
#define LIBRARY_H_

#include <stdlib.h>
#include <stdbool.h>
#include <stdint.h>
#include <string.h>
#include <math.h>

#include <propeller.h>

#ifdef __cplusplus
extern "C" {
#endif

#define HIGH    1
#define LOW     0

#define INPUT   0
#define OUTPUT  1

#define PI          3.1415926535897932384626433832795
#define HALF_PI     1.5707963267948966192313216916398
#define TWO_PI      6.283185307179586476925286766559
#define DEG_TO_RAD  0.017453292519943295769236907684886
#define RAD_TO_DEG  57.295779513082320876798154814105
#define EULER       2.718281828459045235360287471352

#define LSBFIRST    0
#define MSBFIRST    1

// undefine stdlib's abs if encountered
#ifdef abs
#undef abs
#endif

#define min(a,b)                ((a)<(b)?(a):(b))
#define max(a,b)                ((a)>(b)?(a):(b))
#define abs(x)                  ((x)>0?(x):-(x))
#define constrain(amt,low,high) ((amt)<(low)?(low):((amt)>(high)?(high):(amt)))
#define round(x)                ((x)>=0?(long)((x)+0.5):(long)((x)-0.5))
#define radians(deg)            ((deg)*DEG_TO_RAD)
#define degrees(rad)            ((rad)*RAD_TO_DEG)
#define sq(x)                   ((x)*(x))

#define clockCyclesPerMicrosecond()  (_CLKFREQ / 1000000L)
#define clockCyclesToMicroseconds(a) ((a) / clockCyclesPerMicrosecond())
#define microsecondsToClockCycles(a) ((a) * clockCyclesPerMicrosecond())

#define lowByte(w)  ((uint8_t) ((w) & 0xff))
#define highByte(w) ((uint8_t) ((w) >> 8))

#define bitRead(value, bit)            (((value) >> (bit)) & 0x01)
#define bitSet(value, bit)             ((value) |= (1UL << (bit)))
#define bitClear(value, bit)           ((value) &= ~(1UL << (bit)))
#define bitWrite(value, bit, bitvalue) (bitvalue ? bitSet(value, bit) : bitClear(value, bit))

#define INLINE__   static inline
#define Yield__()  __asm__ volatile( "" ::: "memory" )

#ifndef _NOP
#define _NOP()     do { __asm__ volatile ("nop"); } while (0)
#endif

typedef uint8_t     byte;
typedef uint16_t    word;
typedef bool        boolean;

#define bit(b)       (1UL << (b))

void pinMode(int pin, int mode);

void digitalWrite(int pin, int value);
int  digitalRead(int pin);

void delay(unsigned int ms);
void delayMicroseconds(unsigned int us);

void     shiftOut(int32_t dataPin, int32_t clockPin, int32_t bitOrder, uint32_t val);
uint32_t shiftIn(int32_t dataPin, int32_t clockPin, int32_t bitOrder);

void    i2cStart();
void    i2cStop();
int     i2cWrite(uint8_t data);
uint8_t i2cRead(int ack);

void loadOverlay(int n);

#ifdef __cplusplus
} // extern "C"
#endif

class _UARTBase {
public:
    _UARTBase() { }

    virtual void    flush();
    virtual int32_t available();

    virtual int32_t read();
    virtual int32_t read(int32_t ms);
    virtual int32_t readAvailable();

    virtual void    write(int32_t txbyte);
    virtual void    writeBytes(const uint8_t * txbytes, uint32_t length);
};

#endif /* LIBRARY_H_ */
